/*
Copyright (c) 2007-2014 Tasuku HIRAISHI <tasuku@media.kyoto-u.ac.jp>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#define CMD_STARTPROGRAM          0x00
// byte 2-21: File name + '\0'

#define CMD_STOPPROGRAM           0x01

#define CMD_PLAYSOUNDFILE         0x02
// byte 2: Loop?
// byte 3-22: File name + '\0'

#define CMD_PLAYTONE              0x03
// byte 2-3: Frequency[Hz](200-14000)
// byte 4-5: Duration[ms]

#define CMD_SETOUTPUTSTATE        0x04
// byte 2: port(0-2 or 0xff(all ports))
// byte 3: Power(-100(0x9c) -- 100(0x64))
// byte 4: Mode (bitfield)
#define MD_MOTORON   0x01
#define MD_BRAKE     0x02
#define MD_REGURATED 0x04 //Power reglulation according to `Regulation Mode' 
// byte 5: Regulation Mode
#define REGULATION_MODE_IDLE        0x00 // Disables regulation
#define REGULATION_MODE_MOTOR_SPEED 0x01 // Enables speed regulation
#define REGULATION_MODE_MOTOR_SYNC  0x02 // Enables synchronization between any two motors
// byte 6: Turn Ratio(-100(0x9c) -- 100(0x64))
// -- Proportoinal turning ratio for synchronized two motors.
// byte 7: RunState
#define MOTOR_RUN_STATE_IDLE     0x00 // Disables power to specified ports
#define MOTOR_RUN_STATE_RAMPUP   0x10 // Enables automatic ramp-up (increase smoothly) to the SPEED set-point
#define MOTOR_RUN_STATE_RUNNING  0x20 // Enables power to specified port(s) at the SPEED set-point.
#define MOTOR_RUN_STATE_RAMPDOWN 0x40 // Enables automatic ramp-down (decrease smoothly) to SPEED set-point
// byte 8-12: TachoLimit (ulong, 0:forever)
// -- Rotational distance in degrees you want to turn the motor.

#define CMD_SETINPUTMODE          0x05
// byte 2: Input port(0-3)
// byte 3: Sensor Type
#define ST_NO_SENSOR          0x00 // No sensor configured
#define ST_SWITCH             0x01 // NXT or RCX touch sensour
#define ST_TEMPERATURE        0x02 // RCX temperatur sensor
#define ST_REFLECTION         0x03 // RCX light sensor
#define ST_ANGLE              0x04 // RCX rotation sensor
#define ST_LIGHT_ACTIVE       0x05 // NXT light sensor with floodlight enabled
#define ST_LIGHT_INACTIVE     0x06 // NXT light sensor with floodlight disabled
#define ST_SOUND_DB           0x07 // NXT sound sensor; dB scaling
#define ST_SOUND_DBA          0x08 // NXT sound sensor; dBA scaling
#define ST_CUSTOM             0x09 // Unused in NXT programs
#define ST_LOWSPEED           0x0a // I2C digital sensor
#define ST_LOWSPEED_9V        0x0b // I2C digital sensor; 9V power
#define ST_NO_OF_SENSOR_TYPES 0x0c // Unused in NXT programs
// byte 4: Sensor Mode
#define SM_RAWMODE           0x00 // 0-1023
#define SM_BOOLEANMODE       0x20 // 0,1
#define SM_TRANSITIONCNTMODE 0x40 // 0-65535
#define SM_PERIODCOUNTERMODE 0x60 // 0-65535
#define SM_PCTFULLSCALEMODE  0x80 // 0,100
#define SM_CELSIUSMODE       0xa0 // -200, 700 (10th of Celsius)
#define SM_FAHRENHEITMODE    0xc0 // -400, 1580 (10th of Fahrenheit)
#define SM_ANGLESTEPSMODE    0xe0 // 0-65535
#define SM_SLOPEMASK         0x1f // ?
#define SM_MODEMASK          0xe0 // ?

#define CMD_GETOUTPUTSTATE        0x06
// byte 2: Output port(0-2)

#define CMD_GETINPUTVALUES        0x07
// byte 2: Input port(0-3)

#define CMD_RESETINPUTSCALEDVALUE 0x08
// byte 2: Input port(0-3)
#define CMD_MESSAGEWRITE          0x09
// byte 2: Inbox number(0-9)
// byte 3: Message Size
// byte 4-N: Message dat, null termination

#define CMD_RESETMOTORPOSITION    0x0a
// byte 2: Output port (0-2)
// byte 3: Relative?

#define CMD_GETBATTERYLEVEL       0x0b

#define CMD_STOPSOUNDPLAYBACK     0x0c

#define CMD_KEEPALIVE             0x0d

#define CMD_LSGETSTATUS           0x0e
// byte 2: Port (0-3)

#define CMD_LSWRITE               0x0f
// byte 2: Port (0-3)
// byte 3: Tx Data Length[byte]
// byte 4: Rx Data Length[byte]
// byte 5-N: Tx Data

#define CMD_LSREAD                0x10
// byte 2: Port (0-3)

#define CMD_GETCURRENTPROGRAMNAME 0x11

#define CMD_MESSAGEREAD           0x12
// byte 2: Remote Inbox number (0-9)
// byte 3: Local Inbox number (0-9
// byte 4: Remove?
