/*
Copyright (c) 2007-2014 Tasuku HIRAISHI <tasuku@media.kyoto-u.ac.jp>

Pesrmission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

// includes...

#include "fantom/iNXT.h"
#include "fantom/iNXTIterator.h"
#include "fantom/tStatus.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#ifdef _WIN32
#define DllExport __declspec(dllexport)
#else
#define DllExport
#endif


// globally-scoped globals...

// private globals...
static nFANTOM100::tStatus NxtStatus;
static nFANTOM100::iNXT* NxtPtr;

// methods...

int hexchar2int (char c)
{
  if ('0' <= c && c <= '9')
    return c-'0';
  else if ('a' <= c && c<='f')
    return c-'a'+10;
  else if ('A' <= c && c<='F')
    return c-'A'+10;
  else return 0;
}

int hexstr2int (char* str, int n)
{
  int ret=0;
  int scale=1;
  for (int i=n-1; i>=0 ; i--, scale*=16)
    ret += hexchar2int(str[i])*scale;
  return ret;
}

int getcommand (ViUInt8 buffer[], char *cmdstring)
{
  int len = strlen(cmdstring);
  int i;
  for (i=0; i< len/2; i++) {
    buffer[i] = (ViUInt8) hexstr2int(cmdstring+i*2, 2);
    fprintf (stderr, " %02x", buffer[i]);
  }
  fputc('\n', stderr);
  return i*sizeof(ViUInt8);
}


extern "C" {
DllExport int _VI_FUNCC SetupNXT(int bluetooth_timeout)
{
  nFANTOM100::iNXTIterator* nxtIteratorPtr = NULL;
  
  NxtStatus.clear();
  nxtIteratorPtr = nFANTOM100::iNXT::createNXTIterator( bluetooth_timeout?true:false,
							bluetooth_timeout, NxtStatus );
   
  if( NxtStatus.isNotFatal())
    {
      NxtPtr = nxtIteratorPtr->getNXT( NxtStatus );
      nFANTOM100::iNXT::destroyNXTIterator( nxtIteratorPtr );
    }
  return (!NxtStatus.isNotFatal());
}

DllExport char* _VI_FUNCC GetFirmwareVersion (void)
{
  ViUInt8 protocolVersionMajor = 0;
  ViUInt8 protocolVersionMinor = 0;
  ViUInt8 firmwareVersionMajor = 0;
  ViUInt8 firmwareVersionMinor = 0;
  static char ret[20];
  
  // Query the version numbers for the protocol and firmware installed on the NXT.
  NxtPtr->getFirmwareVersion( protocolVersionMajor, protocolVersionMinor,
			      firmwareVersionMajor, firmwareVersionMinor, NxtStatus );
  sprintf (ret, "%d.%d.%d.%d",
	   protocolVersionMajor, protocolVersionMinor,
	   firmwareVersionMajor, firmwareVersionMinor);
  return ret;
}
  
DllExport char* _VI_FUNCC SendCommandToNXT_string (char *commandstr, int res_length)
{
  // This is a direct command to play a tone.
  ViUInt8 directCommandBuffer[64];
  static ViByte ResponseBuffer[64];
  static char str[64*2+1];
  ViUInt32 cmdsize;
  ViUInt32 ressize;
  
  res_length = ((res_length<0)||(res_length>64))? 0: res_length;
 
  cmdsize = getcommand (directCommandBuffer, commandstr);
  ressize = 
    NxtPtr->sendDirectCommand( res_length,
			       reinterpret_cast< ViByte* >( directCommandBuffer ), cmdsize,
			       (res_length)?ResponseBuffer:NULL, res_length,
			       NxtStatus );
  if (res_length) {
    for (int i=0; i<res_length; i++)
      sprintf(str+(i*2), "%02x", ResponseBuffer[i]);
    str[res_length*2]='\0';
    return str;
  } else {
    return "";
  }
}

DllExport int _VI_FUNCC SendCommandToNXT (ViUInt8 *command, int cmdsize, ViByte *response, int ressize )
{
  NxtPtr->sendDirectCommand( ressize,
			     reinterpret_cast< ViByte* >( command ), cmdsize,
			     (ressize)?response:NULL, ressize,
			     NxtStatus );
  return (NxtStatus.isFatal());
}

DllExport void _VI_FUNCC DestroyNXT (void)
{
  // Destroy the NXT object.
  nFANTOM100::iNXT::destroyNXT( NxtPtr );
}

}

// extern "C"
// int _VI_FUNCC main (int argc, char** argv)
// {
//   SetupNXT();
//   if (NxtStatus.isNotFatal())
//   {
//     SendCommandToNXT (argv[1], argc>2);
//     DestroyNXT();
//   }
//   else {
//     fprintf (stderr, "Failed to connect to Mindstorms NXT.\n");
//     return 1;
//   }

//   return (!NxtStatus.isNotFatal());
// }

//////////////////////////////////////////////////
// How to make nxt.dll
// cl -D_MT -MD -nologo -LD -Zi -W3 -Fenxt.dll nxt-transmit.cpp user32.lib gdi32.lib kernel32.lib comctl32.lib comdlg32.lib winmm.lib advapi32.lib msvcrt.lib targets\win32U\i386\msvc71\release\fantom.lib -I includes

// Load nxt.dll on Allegro Common Lisp
// (load "nxt.lsp")
// vfantom.dlliRoboLab2.9ȂǂCXg[ƂĂj
// VistaƊǗ҃[hACLsĂKv
