;; Copyright (c) 2007-2014 Tasuku HIRAISHI <tasuku@media.kyoto-u.ac.jp>
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in
;; all copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
;; THE SOFTWARE.

;; (defpackage :mindstorms-nxt
;;   (:nicknames :nxt)
;;   (:use :cl)
;;   (:export :setup-nxt :destroy-nxt :firmware-version
;;     :send-to-nxt-string :send-to-nxt :wait-until :with-watcher
;;     :play-tone :initialize-player :playing? :play
;;     :keep-alive :battery
;;     :*max-speed*
;;     :motor-reset :motor-brake :motor-on :motor
;;     :motor-state :motor-running?
;;     :light-on :light-off :light :rotation-on :rotation
;;     :temparature-on :temparature :touch-on :touched?
;;     :distance-on :distance :sound-on :sound
;;     :ls-get-status :ls-write :ls-read
;;     ))
;; (in-package :nxt)
;;; define foreign call for "nxt.dll"

#-(or allegro sbcl clisp)
(error "Sorry, this program is supported for for Allegro CL, Steel Bank Common Lisp, or GNU CLISP!")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Foreign call
#+allegro
(progn
  (load "nxt.dll")
  (ff:def-foreign-call (c-setup-nxt "SetupNXT")
      ((bluetooth :int fixnum))
    :returning (:int fixnum))
  (ff:def-foreign-call (c-firmware-version "GetFirmwareVersion") 
      ()
    :returning ((* :char) string))
  (ff:def-foreign-call (c-send-command-to-nxt-string "SendCommandToNXT_string")
      ((str (* :char) string) 
       (ressize :int fixnum))
    :returning ((* :char) string))
  (ff:def-foreign-call (c-send-command-to-nxt "SendCommandToNXT")
      ((command (* :unsigned-char))
       (cmdsize :int fixnum)
       (response (* :unsigned-char))
       (ressize :int fixnum))
    :returning (:int fixnum))
  (ff:def-foreign-call (c-destroy-nxt "DestroyNXT")
      ()
    :returning :void)
  )

#+sbcl
(progn
  (sb-alien:load-shared-object "nxt.dll")
  (sb-alien:define-alien-routine ("SetupNXT" c-setup-nxt)
      sb-alien:int
    (bluetooth sb-alien:int))
  (sb-alien:define-alien-routine ("GetFirmwareVersion" c-firmware-version)
      sb-alien:c-string)
  (sb-alien:define-alien-routine ("SendCommandToNXT_string" c-send-command-to-nxt-string)
      sb-alien:c-string
    (str sb-alien:c-string)
    (ressize sb-alien:int))
  (sb-alien:define-alien-routine ("SendCommandToNXT" c-send-command-to-nxt)
      sb-alien:int
    (command (sb-alien:array sb-alien:unsigned-char 64))
    (cmdsize sb-alien:int)
    (response (sb-alien:array sb-alien:unsigned-char 64))
    (ressize sb-alien:int))
  (sb-alien:define-alien-routine ("DestroyNXT" c-destroy-nxt)
      sb-alien:void)
  )

#+clisp
(progn
  (ffi:def-call-out c-setup-nxt (:name "SetupNXT")
    (:arguments (bt ffi:int))
    (:return-type ffi:int)
    (:language :stdc)
    (:library "nxt.dll"))
  (ffi:def-call-out c-firmware-version (:name "GetFirmwareVersion")
    (:return-type ffi:c-string :none)
    (:language :stdc)
    (:library "nxt.dll"))
  (ffi:def-call-out c-send-command-to-nxt-string
      (:name "SendCommandToNXT_string")
    (:arguments (str ffi:c-string)
                (ressize ffi:int))
    (:return-type ffi:c-string :none)
    (:language :stdc)
    (:library "nxt.dll"))
  (ffi:def-call-out c-send-command-to-nxt (:name "SendCommandToNXT")
    (:arguments (command (ffi:c-ptr (ffi:c-array-max ffi:uint8 64)) :in :alloca)
                (cmdsize ffi:int)
                (response (ffi:c-ptr (ffi:c-array ffi:uint8 64)) :out :alloca)
                (ressize ffi:int))
    (:return-type ffi:int)
    (:language :stdc)
    (:library "nxt.dll"))
  (ffi:def-call-out c-destroy-nxt (:name "DestroyNXT")
    (:return-type nil)
    (:language :stdc)
    (:library "nxt.dll"))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Utils
#+allegro
(progn
  (defmacro uc-aref (unsigned-char-array index)
    `(ff:fslot-value-typed 
      '(:array :unsigned-char) nil ,unsigned-char-array ,index))

  (defun make-unsigned-char-array (array &aux (len (length array)))
    (let ((ret (ff:allocate-fobject `(:array :unsigned-char ,len))))
      (dotimes (i len)
        (setf (uc-aref ret i) (aref array i)))
      ret))

  (defun apply-later (sleep-time func &rest args)
    (mp:process-run-function (symbol-name (gensym "PROC"))
      #'(lambda ()
          (sleep sleep-time)
          (apply func args))))
  (defun funcall-later (sleep-time func &rest args)
    (apply-later sleep-time func args nil))
  )

(defun restrict-num (num min max)
  (or (and min (< num min) min)
      (and max (> num max) max)
      num))

(defun lendian-hexstring (num bytes)
  (with-output-to-string (o-str)
    (labels ((aux (b q r)
               (when (> b 0) 
                 (format o-str "~2,'0X" r)
                 (multiple-value-call 
                     #'aux (1- b) (floor q #x100)))))
      (multiple-value-call #'aux bytes (floor num #x100)))))

(defun lendian-bytearray (num bytes)
  (let ((ret (make-array bytes :element-type 'unsigned-byte)))
    (labels ((aux (b q r)
               (if (> b 0)
                   (progn
                     (setf (aref ret (- bytes b)) r)
                     (multiple-value-call 
                         #'aux (1- b) (floor q #x100)))
                   ret)))
      (multiple-value-call #'aux bytes (floor num #x100)))))

(defun parse-lendian-hexstring (str &optional (start 0) (end (length str)))
  (labels ((aux (i acc)
             (if (>= start i)
                 acc
                 (aux (- i 2)
                      (+ (* #x100 acc)
                         (parse-integer 
                          str :start (- i 2) :end i :radix 16)))))) 
    (aux end 0)))

(defun lendian-to-integer (uc-array start end)
  (labels ((aux (i acc)
             (if (< i start)
                 acc
                 (aux (1- i)
                      (+ (ash acc 8) (#+allegro uc-aref #-allegro aref
                                      uc-array i))))))
    (aux (1- end) 0)))

(defun stream-parse-integer (i-str &key (radix 10))
  (if (let ((ch (peek-char nil i-str nil nil)))
        (and ch (digit-char-p ch radix)))
      (loop
         with ret = 0
         for ch = (peek-char nil i-str nil nil)
         while (and ch (digit-char-p ch radix))
         do (setq ret (+ (digit-char-p (read-char i-str) radix)
                         (* radix ret)))
         finally (return ret))
      nil))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; paremeters
(defvar *cmd-playtone*  #x03)
(defvar *cmd-setoutputstate* #x04)
(defvar *cmd-setinputstate* #x05)
(defvar *cmd-getoutputstate* #x06)
(defvar *cmd-getinputvalues* #x07)
(defvar *cmd-getbatterylevel* #x0b)
(defvar *cmd-keepalive* #x0d)
(defvar *cmd-lsgetstatus* #x0e)
(defvar *cmd-lswrite* #x0f)
(defvar *cmd-lsread* #x10)


(defvar *do*  261.6255)
(defvar *do+* 277.1826) (defvar *re-* *do+*)
(defvar *re*  293.6647)
(defvar *re+* 311.1269) (defvar *mi-* *re+*)
(defvar *mi*  329.6275)
(defvar *fa*  349.2282)
(defvar *fa+* 369.9944) (defvar *sol-* *fa+*)
(defvar *sol* 391.9954)
(defvar *la*  440)
(defvar *la+* 466.1637) (defvar *si-* *la+*)
(defvar *si*  493.8833)

;; (aref *tone-vector* <n>) == a frequency of the <n>th tone
(defparameter *tone-vector*
  (coerce
   (loop 
      for i upfrom -1 to 8
      nconc (mapcar #'(lambda (hz) (* hz (expt 2 i)))
                    (list *do* *do+* *re* *re+*
                          *mi* *fa* *fa+* *sol* *la* *la+* *si*)))
   'vector))
(defparameter *tone-vector-length* (length *tone-vector*))

(defvar *max-speed* 100)

(defvar *check-interval* 1/10)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; wrappers
(defun setup-nxt (&optional bluetooth (bluetooth-timeout 10))
  (assert (integerp bluetooth-timeout))
  (= 0 (c-setup-nxt (if bluetooth bluetooth-timeout 0)))
  )

(defun firmware-version ()
  (c-firmware-version))

(defun send-to-nxt-string (str &optional res-length)
  (c-send-command-to-nxt-string str (if res-length res-length 0)))

#+allegro
(let ((send-nxt-lock (mp:make-process-lock)))
  (defun send-to-nxt (array &optional res-length)
    ;; debug print
    ;;     (progn
    ;;       (format *error-output* "~&>> ")
    ;;       (loop for i from 0 to (1- (length array))
    ;;    do (format *error-output* "~2,'0x " (aref array i)))
    ;;       (fresh-line *error-output*))
    ;;
    (let ((command (make-unsigned-char-array array))
          (cmdsize (length array))
          (response
           (ff:allocate-fobject
            `(:array :unsigned-char
                     ,(if res-length (+ 2 res-length) 1))))
          (ressize (if res-length res-length 0)))
      (mp:with-process-lock (send-nxt-lock)
        (c-send-command-to-nxt
         command cmdsize response ressize))
      ;; debug print
      ;;       (when res-length
      ;;    (format *error-output* "~&<< ")
      ;;    (loop for i from 0 to (1- res-length)
      ;;        do (format *error-output* "~2,'0x " (uc-aref response i)))
      ;;    (fresh-line *error-output*))
      ;;
      response))
  )

#+sbcl
(defun send-to-nxt (array &optional res-length)
  (sb-alien:with-alien ((command (sb-alien:array sb-alien:unsigned-char 64))
                        (response (sb-alien:array sb-alien:unsigned-char 64)))
    (let ((cmdsize (length array))
          (ressize (if res-length res-length 0)))
      (dotimes (i cmdsize)
        (setf (sb-alien:deref command i) (aref array i)))
      (c-send-command-to-nxt
       command cmdsize response ressize)
      (let ((resarray (make-array ressize)))
        (dotimes (i ressize)
          (setf (aref resarray i) (sb-alien:deref response i)))
        resarray))))

#+clisp
(defun send-to-nxt (array &optional res-length)
  (multiple-value-bind (ok response)
      (c-send-command-to-nxt
       array (length array) (if res-length res-length 0))
    (declare (ignore ok))
    response))

(defun destroy-nxt ()
  (c-destroy-nxt))

(defun output-port-num (port)
  (cond
    ((keywordp port)
     (case port
       (:a 0) (:b 1) (:c 2)
       (otherwise nil)))
    ((and (integerp port)
          (<= 0 port 2))
     port)))

(defun input-port-num (port)
  (restrict-num (1- port) 0 3))


;;; Event Handlers

;;; wait-until
(defmacro wait-until (form &optional (interval '*check-interval*))
  (let ((intv-sym (gensym))
        (retv-sym (gensym)))
    `(let ((,intv-sym ,interval)
           (,retv-sym))
       (loop
          (setq ,retv-sym ,form)
          (when ,retv-sym (return ,retv-sym))
          (sleep ,intv-sym)))))

;;; watcher
#+allegro
(defmacro with-watcher1 ((handler &body event) &body body)
  (let ((gate-sym (gensym))
        (cproc-sym (gensym))
        (wproc-sym (gensym)))
    `(let* ((,gate-sym (mp:make-gate t))
            (,cproc-sym mp:*current-process*)
            (,wproc-sym
             (mp:process-run-function (symbol-name (gensym "WATCHER"))
                                      #'(lambda ()
                                          (loop
                                             (mp:process-wait 
                                              "Waiting for interrupt proc."
                                              #'mp:gate-open-p ,gate-sym)
                                             (when ,handler
                                               (mp:close-gate ,gate-sym)
                                               (mp:process-interrupt 
                                                ,cproc-sym
                                                #'(lambda ()
                                                    ,@event
                                                    (mp:open-gate ,gate-sym))))
                                             (sleep *check-interval*))))))
       (unwind-protect
            (progn ,@body)
         (mp:process-kill ,wproc-sym)))))

#+allegro
(defmacro with-watcher (handler-list &body body)
  (if (endp handler-list)
      `(progn ,@body)
      `(with-watcher1 ,(car handler-list)
         (with-watcher ,(cdr handler-list)
           ,@body))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; user functions

;;; play
(defun play-tone (hz duration)
  (setq hz (restrict-num (round hz) 200 14000))
  (setq duration (restrict-num (round duration) 0 65535))
  (send-to-nxt 
   (concatenate 'vector
                `#(,*cmd-playtone*)
                (lendian-bytearray hz 2)
                (lendian-bytearray duration 2))))

(defconstant *octave-init* 4)
(defconstant *beat-init* 4)
(defconstant *tempo-init* 120)

(let (#+allegro (player-lock (mp:make-process-lock))
      (octave 4)
      (beat-default 4)
      (tempo 120))                      ; BPM (Beat per Minute)
  (defun initialize-player ()
    (setq octave *octave-init*)
    (setq beat-default *beat-init*)
    (setq tempo *tempo-init*)
    t)
  
  (defun pitch-name-number (name +or- &optional (oct octave))
    (restrict-num
     (+ (* 11 (- oct 1))
        (position (char-upcase name) "C D EF GA B")
        (case +or- ((#\+) 1) ((#\-) -1) (otherwise 0)))
     0 (1- *tone-vector-length*)))
  
  (defun beat-duration (beat time-dot &optional (tp tempo))
    (/ (* 60 4 1000 (if time-dot 1.5 1)) tp beat))
  
  (defun get-mml-action (string &optional (start 0) &aux index)
    (with-input-from-string (i-str string :index index :start start)
      (let ((ch (read-char i-str nil nil)))
        (if ch
            (case (char-upcase ch)
              ;; Musical notations
              ((#\C #\D #\E #\F #\G #\A #\B)
               (let* ((sharp
                       (when (eql #\+ (peek-char nil i-str nil nil))
                         (read-char i-str)))
                      (flat
                       (when (eql #\- (peek-char nil i-str nil nil))
                         (read-char i-str)))
                      (beat
                       (let ((tmp (stream-parse-integer i-str)))
                         (if tmp tmp beat-default)))
                      (time-dot
                       (when (eql #\. (peek-char nil i-str nil nil))
                         (read-char i-str)))
                      (duration (beat-duration beat time-dot))
                      (pit-n (pitch-name-number ch (or sharp flat)))
                      (hz (aref *tone-vector* pit-n)))
                 #'(lambda ()
                     (play-tone hz (- duration 5))
                     (sleep (/ duration 1000))
                     (funcall (get-mml-action string index)))
                 ))
              ;; Rest
              ((#\R)
               (let* ((beat
                       (let ((tmp (stream-parse-integer i-str)))
                         (if tmp tmp beat-default)))
                      (time-dot
                       (when (eql #\. (peek-char nil i-str nil nil))
                         (read-char i-str)))
                      (duration (beat-duration beat time-dot)))
                 #'(lambda ()
                     (sleep (/ duration 1000))
                     (funcall (get-mml-action string index)))))
              ;; Octave
              ((#\O)
               (let ((new-oct
                      (restrict-num
                       (let ((tmp (stream-parse-integer i-str)))
                         (if tmp tmp 4))
                       1 8)))
                 #'(lambda ()
                     (setq octave new-oct)
                     (funcall (get-mml-action string index)))
                 ))
              ;; Length
              ((#\L)
               (let* ((new-length
                       (restrict-num
                        (let ((tmp (stream-parse-integer i-str)))
                          (if tmp tmp 4))
                        1 64)))
                 #'(lambda ()
                     (setq beat-default new-length)
                     (funcall (get-mml-action string index)))
                 ))
              ;; Tempo
              ((#\T)
               (let* ((new-tempo
                       (restrict-num
                        (let ((tmp (stream-parse-integer i-str)))
                          (if tmp tmp 4))
                        1 500)))
                 #'(lambda ()
                     (setq tempo new-tempo)
                     (funcall (get-mml-action string index)))
                 ))
              (otherwise
               (error "MML syntax error.")))
            #'(lambda () t)))))

  (defun playing? ()
    #+allegro (mp:process-lock-locker player-lock)
    )

  (defun play (string)
    #+allegro
    (mp:process-run-function (symbol-name (gensym "PLAYER"))
                             #'(lambda ()
                                 (mp:with-process-lock (player-lock)
                                   (funcall (get-mml-action string)))))
    #+(or clisp sbcl)
    (funcall (get-mml-action string)) )
  )

;;; power management
(defun keep-alive (&optional get-sleep-time)
  (let ((ret (send-to-nxt `#(,*cmd-keepalive*)
                          (if get-sleep-time 6 nil))))
    (if get-sleep-time
        (lendian-to-integer ret 2 6)
        -1)))

(defun battery ()
  (lendian-to-integer
   (send-to-nxt `#(,*cmd-getbatterylevel*) 4)
   2 4))

;;; motor
(defun motor-reset ()
  (dolist (pt '(:a :b :c))
    (motor pt 1)
    (motor pt :brake)))

(defun motor-brake (portnum)
  (send-to-nxt
   (concatenate 'vector
                `#(,*cmd-setoutputstate*)
                (lendian-bytearray portnum 1)
                #(#x00 #x02 #x00 #x00 #x00 #x00 #x00 #x00 #x00)))
  '(:brake))

(defun motor-on (portnum speed &key (distance nil))
  (cond
    ((eql 0 distance)
     (motor-brake portnum))
    ((and (numberp distance)
          (< distance 0))
     (motor-on portnum (- speed) :distance (- distance)))
    (t
     (send-to-nxt
      (concatenate 'vector
                   `#(,*cmd-setoutputstate*)
                   (lendian-bytearray portnum 1)
                   (lendian-bytearray speed 1)
                   #(#x07 #x01 #x00 #x20)
                   (if (numberp distance)
                       (lendian-bytearray distance 4)
                       #(#x00 #x00 #x00 #x00))))
     (list speed distance))))

(defun motor (port speed &rest args)
  (let ((portnum (output-port-num port))
        (operate
         (cond
           ((keywordp speed)
            (case speed
              (:max-speed *max-speed*)
              (:off 0)
              (:brake :brake)
              (otherwise nil)))
           ((numberp speed)
            (restrict-num (round speed)
                          (- *max-speed*) *max-speed*)))))
    (unless portnum
      (error "Unknown port ~S." port))
    (cond
      ((numberp operate)
       (apply #'motor-on portnum operate args))
      ((eq :brake operate)
       (motor-brake portnum))
      (t
       (error "Unknown operate ~S." operate)))))

(defun motor-state (port)
  (let* ((portnum (output-port-num port))
         (reply 
          (send-to-nxt
           (concatenate 'vector
                        `#(,*cmd-getoutputstate*)
                        (lendian-bytearray portnum 1))
           24)))
    `((:status            . ,(lendian-to-integer reply 1 2))
      (:port              . ,(lendian-to-integer reply 2 3))
      (:power-set         . ,(lendian-to-integer reply 3 4))
      (:mode              . ,(lendian-to-integer reply 4 5))
      (:reg-mode          . ,(lendian-to-integer reply 5 6))
      (:turn-ratio        . ,(lendian-to-integer reply 6 7))
      (:runstate          . ,(lendian-to-integer reply 7 8))
      (:tacho-limit       . ,(lendian-to-integer reply 8 12))
      (:tacho-count       . ,(lendian-to-integer reply 12 16))
      (:block-tacho-count . ,(lendian-to-integer reply 16 20))
      (:rotation-count    . ,(lendian-to-integer reply 20 24))
      )))

(defun motor-running? (port &optional state)
  (< 0 (cdr (assoc :runstate
                   (or state (motor-state port))))))

;;; sensor (general)
(defun set-input-port (port sensor-type sensor-mode)
  (let ((portnum (input-port-num port)))
    (send-to-nxt
     (concatenate 'vector
                  `#(,*cmd-setinputstate*)
                  (lendian-bytearray portnum 1)
                  (lendian-bytearray sensor-type 1)
                  (lendian-bytearray sensor-mode 1))))
  port)

(defun get-input-port (port &optional (start 11) (end 13))
  (let ((portnum (input-port-num port)))
    (lendian-to-integer
     (send-to-nxt
      (concatenate 'vector
                   `#(,*cmd-getinputvalues*)
                   (lendian-bytearray portnum 1))
      15)
     start end)))

;;; light sensor
(defun light-on (port)
  (set-input-port port #x05 #x00))

(defun light-off (port)
  (set-input-port port #x06 #x00))

(defun light (port)
  (get-input-port port))

;;; rotation sensor
(defun rotation-on (port)
  (set-input-port port #x04 #xe0))

(defun rotation (port)
  (get-input-port port))

;;; temparature sensor
(defun temparature-on (port &optional (scaling :celcius))
  (set-input-port 
   port #x04
   (case scaling
     ((:celcius)    #xa0)
     ((:fahrenheit) #xc0)
     (otherwise
      (error "Invalid scaling mode ~S." scaling)))
   ))

(defun temparature (port)
  (get-input-port port))

;;; touch sensor
(defun touch-on (port)
  (set-input-port port #x01 #x20))

(defun touched? (port)
  (> (get-input-port port)
     0))


;;; ultrasonic sensor
(defun distance-on (port)
  (let ((portnum (set-input-port port #x0b ; lowspeed_9V
                                 #x00)))
    (do ((ls-state (ls-get-status port) (ls-get-status port)))
        ((numberp ls-state)
         (when (> ls-state 0)
           (ls-read 4))
         (ls-write port #(2 #x41 2) 0)  ; read command state
         portnum)
      (unless (eq :err-comm-chan-not-ready ls-state)
        (return (values nil ls-state))))))

(defun distance (port)
  (ls-write port #(2 #x42) 1)           ; read measurement byte0
  ;; retry until the reply can be got
  (do ((ls-state (ls-get-status port) (ls-get-status port)))
      ((and (numberp ls-state)
            (> ls-state 0))
       (car (ls-read port)))
    (unless (or (eq :stat-comm-pending ls-state)
                (numberp ls-state))
      (return (values nil ls-state)))))

;;; sound sensor
(defun sound-on (port &optional (scaling :dba))
  (set-input-port 
   port 
   (case scaling
     ((:db)  #x07)
     ((:dba) #x08)
     (otherwise
      (error "Invalid scaling mode ~S." scaling)))
   #x00))

(defun sound (port)
  (get-input-port port))


;;; LS communication with NXT
(defun ls-get-status (port)
  (let* ((portnum (input-port-num port))
         (reply
          (send-to-nxt (concatenate 'vector
                         `#(,*cmd-lsgetstatus*)
                         (lendian-bytearray portnum 1))
                       3))
         (stat (lendian-to-integer reply 1 2)))
    (case stat
      ((#x00) (lendian-to-integer reply 2 3))
      ((#x20) :stat-comm-pending)
      ((#xdd) :err-comm-bus-err)
      ((#xdf) :err-comm-chan-invalid)
      ((#xe0) :err-comm-chan-not-ready)
      (otherwise `(:unknown ,stat)))))

(defun ls-write (port write-bytes rx-data-length)
  (let ((portnum (input-port-num port)))
    (send-to-nxt (concatenate 'vector
                              `#(,*cmd-lswrite*)
                              (lendian-bytearray portnum 1)
                              (lendian-bytearray (length write-bytes) 1)
                              (lendian-bytearray rx-data-length 1)
                              write-bytes)
                 2)))

(defun ls-read (port)
  (let* ((rx-length
          (wait-until
           (let ((tmp (ls-get-status port)))
             (and (numberp tmp) (> tmp 0) tmp))))
         (portnum (input-port-num port))
         (reply
          (send-to-nxt (concatenate 'vector
                                    `#(,*cmd-lsread*)
                                    (lendian-bytearray portnum 1))
                       19)))
    (loop for i from 3 to (+ 2 rx-length)
        collect (#+allegro uc-aref #-allegro aref
                 reply i))))
