;; Copyright (c) 2007-2014 Tasuku HIRAISHI <tasuku@media.kyoto-u.ac.jp>
;;
;; Permission is hereby granted, free of charge, to any person obtaining a copy
;; of this software and associated documentation files (the "Software"), to deal
;; in the Software without restriction, including without limitation the rights
;; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
;; copies of the Software, and to permit persons to whom the Software is
;; furnished to do so, subject to the following conditions:
;;
;; The above copyright notice and this permission notice shall be included in
;; all copies or substantial portions of the Software.
;;
;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
;; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
;; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
;; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
;; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
;; THE SOFTWARE.

(defvar *speed* 20)

(defun all-stop ()
  (dolist (pt '(:a :b :c))
    (motor pt :brake)))

(defun spin (&optional (degree t) &key (speed *speed*) (wait nil) 
	     &aux distance)
  (cond
   ((numberp degree) (setq distance (* 2 degree)))
   (degree (setq distance nil))
   (t      (setq speed (- speed))
	   (setq distance nil)))
  (dolist (pt '(:b :c))
    (motor pt (if (eq :b pt) speed (- speed))
	   :distance distance))
  (when wait
    (wait-until (not (motor-running? :b))))
  )

(defun turn (&optional (degree t) &key (speed *speed*) (wait nil) 
	     &aux distance port)
  (cond
   ((numberp degree) (setq distance (* 2 degree))
		     (setq port (if (> degree 0) :b :c)))
   (degree (setq distance nil)
	   (setq port :b))
   (t      (setq distance nil)
	   (setq port :c)))
  (dolist (pt '(:b :c))
    (if (eq port pt)
	(motor pt speed :distance distance)
      (motor pt 0)))
  (when wait
    (wait-until (not (motor-running? port))))
  )

(defun go-forward (&optional (distance t) &key (speed *speed*) (wait nil))
  (when (eq nil distance) (setq speed (- speed)))
  (dolist (pt '(:b :c))
    (motor pt speed :distance distance))
  (when wait
    (wait-until (not (motor-running? :b))))
  )

(defun rover ()
  (distance-on 4)
  ;; PrgmƊJn
  (unwind-protect
      (loop for i from 1 to 10
	  do 
	    ;;(unless (playing?) (play "T200OD2CDFD2FGO5COA2D2"))
	    ;; Oi
	    (go-forward)
	    (wait-until (< (distance 4) 25))
	    ;; ɂԂ
	    ;; Ԃ
	    (go-forward nil)
	    (sleep 0.5)
	    ;; _ōEǂ炩ɋȂ
	    (motor (if (= 0 (random 2)) :b :c) *speed*)
	    (sleep 0.5))
    (all-stop))
  )

(defun go-and-back ()
  (distance-on 4)
  (unwind-protect
      (with-watcher (((< (distance 4) 40)
		      (print t))
		     ((not (playing?))
		      (play "CDEFGABR")))
	(loop
	  (go-forward)
	  (sleep 1)
	  (go-forward nil)
	  (sleep 1)
	  ))
    (all-stop)))


;;
(defun wall ()
  (motor-reset)
  (touch-on 1)
  (distance-on 4)
  (unwind-protect
      (let ((quit nil))
	(with-watcher
	    (((< (distance 4) 35)
	      (go-forward nil)
	      (sleep 1/2)
	      (spin nil)
	      (sleep 3/4)
	      (all-stop))
	     ((touched? 1)
	      (setq quit t)))
	  (until quit
	    (go-forward)
	    (sleep 1)
	    (all-stop)
	    (turn)
	    (sleep 1/4))))
    (all-stop)))
